@@ -1,7 +1,7 @@
 // qsamplerInstrumentForm.cpp
 //
 /****************************************************************************
-   Copyright (C) 2003-2009, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2003-2010, rncbc aka Rui Nuno Capela. All rights reserved.
    Copyright (C) 2007, Christian Schoenebeck
 
    This program is free software; you can redistribute it and/or
@@ -27,8 +27,9 @@
 #include "qsamplerChannel.h"
 #include "qsamplerMainForm.h"
 
-#include <QFileDialog>
 #include <QMessageBox>
+#include <QPushButton>
+#include <QFileDialog>
 
 // Needed for lroundf()
 #include <math.h>
@@ -50,7 +51,7 @@ namespace QSampler {
 // QSampler::InstrumentForm -- Instrument map item form implementation.
 //
 
-InstrumentForm::InstrumentForm ( QWidget* pParent )
+InstrumentForm::InstrumentForm ( QWidget *pParent )
 	: QDialog(pParent)
 {
 	m_ui.setupUi(this);
@@ -96,11 +97,11 @@ InstrumentForm::InstrumentForm ( QWidget* pParent )
 	QObject::connect(m_ui.LoadModeComboBox,
 		SIGNAL(activated(int)),
 		SLOT(changed()));
-	QObject::connect(m_ui.OkPushButton,
-		SIGNAL(clicked()),
+	QObject::connect(m_ui.DialogButtonBox,
+		SIGNAL(accepted()),
 		SLOT(accept()));
-	QObject::connect(m_ui.CancelPushButton,
-		SIGNAL(clicked()),
+	QObject::connect(m_ui.DialogButtonBox,
+		SIGNAL(rejected()),
 		SLOT(reject()));
 }
 
@@ -230,10 +231,6 @@ void InstrumentForm::setup ( Instrument *pInstrument )
 	// Done.
 	m_iDirtySetup--;
 	stabilizeForm();
-
-	// Done.
-	m_iDirtySetup--;
-	stabilizeForm();
 }
 
 
@@ -261,10 +258,20 @@ void InstrumentForm::openInstrumentFile (void)
 
 	// FIXME: the instrument file filters should be restricted,
 	// depending on the current engine.
+	const QString& sEngineName = m_ui.EngineNameComboBox->currentText().toUpper();
+	QStringList filters;
+	if (sEngineName.contains("GIG"))
+		filters << tr("GIG Instrument files") + " (*.gig *.dls)";
+	if (sEngineName.contains("SFZ"))
+		filters << tr("SFZ Instrument files") + " (*.sfz)";
+	if (sEngineName.contains("SF2"))
+		filters << tr("SF2 Instrument files") + " (*.sf2)";
+	const QString& filter = filters.join(";;");
+
 	QString sInstrumentFile = QFileDialog::getOpenFileName(this,
 		QSAMPLER_TITLE ": " + tr("Instrument files"), // Caption.
-		pOptions->sInstrumentDir,                 // Start here.
-		tr("Instrument files") + " (*.gig *.dls)" // Filter (GIG and DLS files)
+		pOptions->sInstrumentDir, // Start here.
+		filter                    // File filter.
 	);
 
 	if (sInstrumentFile.isEmpty())
@@ -364,7 +371,7 @@ void InstrumentForm::reject (void)
 	bool bReject = true;
 
 	// Check if there's any pending changes...
-	if (m_iDirtyCount > 0 && m_ui.OkPushButton->isEnabled()) {
+	if (m_iDirtyCount > 0) {
 		switch (QMessageBox::warning(this,
 			QSAMPLER_TITLE ": " + tr("Warning"),
 			tr("Some channel settings have been changed.\n\n"
@@ -402,16 +409,15 @@ void InstrumentForm::changed (void)
 // Stabilize current form state.
 void InstrumentForm::stabilizeForm (void)
 {
-	bool bValid =
-		!m_ui.NameLineEdit->text().isEmpty() &&
-		m_ui.EngineNameComboBox->currentIndex() >= 0 &&
-		m_ui.EngineNameComboBox->currentText() !=
-		Channel::noEngineName();
+	bool bValid = !m_ui.NameLineEdit->text().isEmpty()
+		&& m_ui.EngineNameComboBox->currentIndex() >= 0
+		&& m_ui.EngineNameComboBox->currentText() != Channel::noEngineName();
 
 	const QString& sPath = m_ui.InstrumentFileComboBox->currentText();
 	bValid = bValid && !sPath.isEmpty() && QFileInfo(sPath).exists();
 
-	m_ui.OkPushButton->setEnabled(m_iDirtyCount > 0 && bValid);
+	m_ui.DialogButtonBox->button(
+		QDialogButtonBox::Ok)->setEnabled(m_iDirtyCount > 0 && bValid);
 }
 
 } // namespace QSampler
